@@ -67,7 +67,7 @@ sub init {
 	$self-> {header} = $self-> insert($profile{headerClass} =>
 		name     => 'Header',
 		origin   => [ $bw, $sz[1] - $bw - $hh],
-		size     => [ $sz[0] - $bw * 2 + 1, $hh],
+		size     => [ $sz[0] - $bw * 2, $hh],
 		vertical => 0,
 		growMode => gm::Ceiling,
 		items    => $profile{headers},
@@ -326,7 +326,7 @@ sub autoRecalc {($#_)?$_[0]->set_auto_recalc ($_[1]):return $_[0]->{autoRecalc}
 
 =head1 NAME
 
-Prima::DetailedOutline - a multi-column outline viewer with controlling
+Prima::DetailedOutline - multi-column outline viewer with controlling
 header widget.
 
 =head1 SYNOPSIS
@@ -356,6 +356,15 @@ header widget.
 
 =for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/detailedoutline.gif">
 
+  my $l = Prima::DetailedOutline->new(
+     style => 'triangle',
+     ...
+  );
+
+=for podview <img src="detailedoutline2.gif">
+
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/detailedoutline2.gif">
+
 =head1 DESCRIPTION
 
 Prima::DetailedOutline combines the functionality of Prima::OutlineViewer
@@ -363,9 +372,10 @@ and Prima::DetailedList.
 
 =head1 API
 
-This class inherits all the properties, methods, and events of Prima::OutlineViewer
-(primary ancestor) and Prima::DetailedList (secondary ancestor).  One new property
-is introduced, and one property is different enough to warrant mention.
+This class inherits all the properties, methods, and events of
+Prima::OutlineViewer (primary ancestor) and Prima::DetailedList (secondary
+ancestor).  One new property C<autoRecalc> is introduced, and the C<items>
+property is different enough to warrant the mention.
 
 =head2 Methods
 
@@ -375,7 +385,7 @@ is introduced, and one property is different enough to warrant mention.
 
 Each item is represented by an arrayref with either one or two elements.  The
 first element is the item data, an arrayref of text strings to display.  The
-second element, if present, is an arrayref of child items.
+second element, if present, is an arrayref of children.
 
 When using the node functionality inherited from Prima::OutlineViewer, the
 item data (that is, the arrayref of text strings) is the first element of the
