@@ -5,6 +5,8 @@ use strict;
 use warnings;
 use Prima;
 
+sub reset_matrix { shift->set_matrix( $Prima::matrix::identity ) }
+
 sub rect3d
 {
 	my ( $self, $x, $y, $x1, $y1, $width, $lColor, $rColor, $backColor) = @_;
@@ -324,7 +326,8 @@ sub stroke_img_primitive
 	return 1 if $self->rop == rop::NoOper;
 	return 1 if $self->linePattern eq lp::Null && $self->rop2 == rop::NoOper;
 
-	my $path = $self->new_path;
+	my $single_pixel = $self->lineWidth < 1.5;
+	my $path = $self->new_path( subpixel => !$single_pixel);
 	my $matrix = $self-> get_matrix;
 	$path->matrix( $matrix );
 	$path->$request(@_);
@@ -333,10 +336,10 @@ sub stroke_img_primitive
 	return $self->graphic_context( sub {
 		# paths produce floating point coordinates and line end arcs,
 		# here we need internal pixel-wise plotting
-		$self-> set_matrix( $Prima::matrix::identity );
+		$self-> reset_matrix;
 		$self-> lineWidth(0);
 		return $path->stroke;
-	}) if $self->lineWidth < 1.5;
+	}) if $single_pixel;
 
 	my %widen;
 	my $method;
@@ -350,53 +353,30 @@ sub stroke_img_primitive
 	my $region2 = $self->region;
 	my $path2   = $path->widen(%widen);
 	my $region1 = $path2->region(fm::Winding | fm::Overlay);
-	my @box = $region1->box;
-	$box[$_+2] += $box[$_] for 0,1;
+	my @rect    = $region1->rect;
 
 	return unless $self->graphic_context_push;
 	$self->fillPattern(fp::Solid);
-	$self->set_matrix( $Prima::matrix::identity );
+	$self->reset_matrix;
 	if ( $self-> rop2 == rop::CopyPut && $self->linePattern ne lp::Solid && $self->linePattern ne lp::Null ) {
+		my $c = $self->color;
 		$self->color($self->backColor);
 		my $path3 = $path->widen( linePattern => lp::Solid );
 		my $region3 = $path3->region;
 		$region3->combine( $region1, rgnop::Diff);
 		$region3->combine($region2, rgnop::Intersect) if $region2;
 		$self->region($region3);
-		$ok = $self->bar(@box);
+		$ok = $self->bar(@rect);
+		$self->color($c);
 	}
 
 	$region1->combine($region2, rgnop::Intersect) if $region2;
 	$self->region($region1);
-	$ok &&= $self->$method(@box);
+	$ok &&= $self->$method(@rect);
 	$self->graphic_context_pop;
 	return $ok;
 }
 
-sub fill_img_primitive
-{
-	my ( $self, $request, @p ) = @_;
-
-	return unless $self->graphic_context_push;
-	my $path = $self->new_path;
-	$path->matrix( $self->get_matrix);
-	$path->$request(@p);
-
-	my $region1 = $path->region( $self-> fillMode);
-	my $region2 = $self->region;
-	$region1->combine($region2, rgnop::Intersect) if $region2;
-	my @box = $region1->box;
-	$box[$_+2] += $box[$_] for 0,1;
-	$self->region($region1);
-
-	$self->set_matrix($Prima::matrix::identity);
-	my $ok = $self->bar(@box);
-
-	$self->graphic_context_pop;
-
-	return $ok;
-}
-
 sub stroke_imgaa_primitive
 {
 	my ( $self, $request ) = (shift, shift);
@@ -404,50 +384,40 @@ sub stroke_imgaa_primitive
 	my $lp = $self->linePattern;
 	return 1 if $lp eq lp::Null && $self->rop2 == rop::NoOper;
 
-	my $aa = $self->new_aa_surface;
-	return 0 unless $aa->can_aa;
-
-	my $path = $self->new_path;
-	$path->set_matrix( $self-> matrix );
+	my $path = $self->new_path( subpixel => 1 );
 	$path->$request(@_);
-	$path = $path->widen(
+	my $path2 = $path->widen(
 		linePattern => ( $lp eq lp::Null) ? lp::Solid : $lp
 	);
 
 	return unless $self->graphic_context_push;
+
 	$self->fillPattern(fp::Solid);
 	$self->fillMode(fm::Winding | fm::Overlay);
-	$self->set_matrix($Prima::matrix::identity);
-	$self->color($self->backColor) if $lp eq lp::Null;
-	my $ok = 1;
-	for ($path->points(fill => 1)) {
-		$ok &= $aa->fillpoly($_);
-		last unless $ok;
+	if ( $self-> rop2 == rop::CopyPut && $self->linePattern ne lp::Solid && $self->linePattern ne lp::Null ) {
+		my $c = $self->color;
+		$self->color($self->backColor);
+		my $path3 = $path->widen( linePattern => lp::Solid);
+		$path3->fill;
+		$self->color($c);
 	}
+
+	$self->color($self->backColor) if $lp eq lp::Null;
+	my $ok = $path2->fill;
 	$self->graphic_context_pop;
 	return $ok;
 }
 
-sub fill_imgaa_primitive
+sub execute_img_primitive
 {
 	my ( $self, $request ) = (shift, shift);
+	my $ps = $self->get_paint_state;
+	return $self->stroke_primitive($request, @_) if $ps == ps::Enabled;
+	return 1 if $ps == ps::Information;
 
-	my $aa = $self->new_aa_surface;
-	return 0 unless $aa->can_aa;
-
-	my $ok = 1;
-	my $path = $self->new_path;
-	my $m = $self->get_matrix;
-	$path->matrix($m);
-	$path->$request(@_);
-	$self->matrix($Prima::matrix::identity);
-	for ($path->points(fill => 1)) {
-		next if $aa->fillpoly($_);
-		$ok = 0;
-		last;
-	}
-	$self->set_matrix($Prima::matrix::identity);
-	return $ok;
+	my $suffix = $self->antialias ? 'aa' : '';
+	my $act .= 'stroke_img' . $suffix . '_primitive';
+	return $self->$act($request, @_);
 }
 
 sub stroke_primitive
@@ -457,36 +427,49 @@ sub stroke_primitive
 	my $lp = $self->linePattern;
 	return 1 if $lp eq lp::Null && $self->rop2 == rop::NoOper;
 
-	my $path = $self->new_path;
+	my $single_pixel = !$self->antialias && $self->alpha == 255 && $self->lineWidth < 1.5;
+	my $path = $self->new_path( subpixel => !$single_pixel);
+	$path->matrix( $self-> matrix );
 	$path->$request(@_);
 
 	return $self->graphic_context( sub {
 		$self-> lineWidth(0);
+		$self-> matrix-> identity;
 		return $path->stroke;
-	} ) if !$self->antialias && $self->alpha == 255 && $self->lineWidth < 1.5;
+	} ) if $single_pixel;
 
-	$path = $path->widen(
+	my $path2 = $path->widen(
 		linePattern => ( $lp eq lp::Null) ? lp::Solid : $lp,
 		(!$self->antialias && $self->lineWidth == 0) ? (lineWidth => 1) : (),
 	);
+	$path2->subpixel( $self-> antialias );
 	return unless $self->graphic_context_push;
+	$self-> matrix-> identity;
 	$self->fillPattern(fp::Solid);
 	$self->fillMode(fm::Winding | fm::Overlay);
+	if ( $self-> rop2 == rop::CopyPut && $self->linePattern ne lp::Solid && $self->linePattern ne lp::Null ) {
+		my $c = $self->color;
+		$self->color($self->backColor);
+		my $path3 = $path->widen(
+			linePattern => lp::Solid,
+			(!$self->antialias && $self->lineWidth == 0) ? (lineWidth => 1) : (),
+		);
+		$path3->subpixel( $self-> antialias );
+		$path3->fill;
+		$self->color($c);
+	}
 	$self->color($self->backColor) if $lp eq lp::Null;
-	my $ok = $path->fill;
+	my $ok = $path2->fill;
 	$self->graphic_context_pop;
 	return $ok;
 }
 
 sub fill_primitive
 {
-	my ( $self, $request ) = (shift, shift);
+	my ( $self, $request, @p ) = @_;
 	my $path = $self->new_path;
-	$path->$request(@_);
-	for ($path->points(fill => 1)) {
-		return 0 unless $self->fillpoly($_);
-	}
-	return 1;
+	$path->$request(@p);
+	return $path->fill;
 }
 
 sub text_shape_out
@@ -528,7 +511,7 @@ sub text_wrap_shape
 	} else {
 		$shaped = $self-> text_shape( $text, %opt );
 	}
-	return $self->text_wrap( $text, $width // -1, $opt, delete($opt{tabs}) // 8) unless $shaped;
+	return $self->text_wrap( $text, $width // -1, $opt & ~tw::ReturnGlyphs, delete($opt{tabs}) // 8) unless $shaped;
 	my $ret    = $self-> text_wrap( $text, $width // -1, $opt, delete($opt{tabs}) // 8, 0, -1, $shaped);
 
 	if (( my $justify = delete $opt{justify} ) && $ret && @$ret ) {
@@ -714,7 +697,7 @@ sub render_pattern
 
 Prima::Drawable::Basic
 
-=head1 NAME
+=head1 DESCRIPTION
 
 Basic drawing routines for Prima::Drawable
 
