@@ -8,15 +8,16 @@
 #
 package Prima::ScrollBar;
 use vars qw(@ISA @stdMetrics);
-@ISA = qw(Prima::Widget Prima::MouseScroller);
-
-@stdMetrics = Prima::Application-> get_default_scrollbar_metrics;
-my $win32 = (Prima::Application-> get_system_info-> {apc} == apc::Win32);
-
 use strict;
 use warnings;
-use Prima::Const;
-use Prima::IntUtils;
+use Prima;
+use base qw(
+	Prima::Widget
+	Prima::Widget::Fader
+	Prima::Widget::MouseScroller
+);
+
+@stdMetrics = Prima::Application-> get_default_scrollbar_metrics;
 
 sub profile_default
 {
@@ -27,13 +28,13 @@ sub profile_default
 		growMode      => gm::GrowHiX,
 		height        => $stdMetrics[1],
 		min           => 0,
-		minThumbSize  => 21 * $::application->uiScaling,
+		minThumbSize  => int( 21 * $::application->uiScaling + .5),
 		max           => 100,
 		pageStep      => 10,
 		partial       => 10,
 		selectable    => 0,
 		step          => 1,
-		style         => ( $win32 ? 'xp' : 'os2' ),
+		syncPaint     => 1,
 		tabStop       => 0,
 		value         => 0,
 		vertical      => 0,
@@ -45,6 +46,7 @@ sub profile_default
 {
 my %RNT = (
 	%{Prima::Widget-> notification_types()},
+	%{Prima::Widget::Fader-> notification_types()},
 	Track => nt::Default,
 );
 
@@ -70,9 +72,9 @@ sub init
 	my %profile = $self-> SUPER::init(@_);
 	for ( qw( b1 b2 tab left right)) { $self-> { $_}-> { pressed} = 0; }
 	for ( qw( b1 b2 tab)) { $self-> { $_}-> { enabled} = 0; }
-	for ( qw( autoTrack minThumbSize vertical min max step pageStep whole partial value style))
+	for ( qw( autoTrack minThumbSize vertical min max step pageStep whole partial value ))
 		{ $self-> {$_} = 1; }
-	for ( qw( autoTrack minThumbSize vertical min max step pageStep whole partial value style))
+	for ( qw( autoTrack minThumbSize vertical min max step pageStep whole partial value ))
 		{ $self-> $_( $profile{ $_}); }
 	$self-> {suppressNotify} = undef;
 	$self-> reset;
@@ -112,28 +114,48 @@ sub on_size
 
 sub draw_pad
 {
-	my ( $self, $canvas, $part, $base_color, $cache) = @_;
+	my ( $self, $canvas, $part, $base_color, $skin) = @_;
 
-	if ( defined $self->{prelight} && $self->{prelight} eq $part && $self->enabled ) {
-		$base_color = $self-> prelight_color($base_color, 1.25);
+	if ( $skin eq 'flat') {
+		my $hbc = $self-> hiliteBackColor;
+		my $clr;
+		if ( $self->{$part}->{pressed}) {
+			$clr = $hbc;
+		} elsif ( $self->enabled) {
+			my $pct = 0.33;
+			$pct *= 1 + ( $self-> fader_current_value // 1 )
+				if ( $self->{prelight} // '') eq $part;
+			$clr = cl::blend(
+				$self->map_color( $base_color ),
+				$self->map_color( $hbc ),
+				$pct
+			);
+		} else {
+			$clr = $base_color;
+		}
+		$canvas-> color( $clr );
+		$canvas-> bar( @{$self-> {$part}-> {rect}} );
+	} else {
+		if ( defined $self->{prelight} && $self->{prelight} eq $part && $self->enabled ) {
+			my $hc = $self-> prelight_color($base_color, 1.25);
+			$base_color = cl::blend( $base_color, $hc, $self->fader_current_value // 1);
+		}
+		my @palette = ( $base_color, $self->dark3DColor );
+		my @spline  = ( 1, 0 );
+		unless ($self-> { vertical } ) {
+			@palette = reverse @palette;
+			@spline  = reverse @spline;
+		}
+		$self-> rect_bevel( $canvas, @{$self-> {$part}-> {rect}},
+			width   => 2,
+			concave => $self->{$part}->{pressed},
+			fill    => $canvas->new_gradient(
+				palette   => \@palette,
+				spline    => \@spline,
+				vertical  => $self->{vertical},
+			),
+		);
 	}
-
-	my @palette = ( $base_color, $self->dark3DColor );
-	my @spline  = ( 1, 0 );
-	unless ($self-> { vertical } ) {
-		@palette = reverse @palette;
-		@spline  = reverse @spline;
-	}
-
-	$self-> rect_bevel( $canvas, @{$self-> {$part}-> {rect}},
-		width   => 2,
-		concave => $self->{$part}->{pressed},
-		fill    => $canvas->new_gradient(
-			palette   => \@palette,
-			spline    => \@spline,
-			vertical  => $self->{vertical},
-		),
-	);
 }
 
 sub fix_triangle
@@ -158,10 +180,12 @@ sub fix_triangle
 sub on_paint
 {
 	my ($self,$canvas) = @_;
+	my $skin = $self->skin;
 	my @clr;
 	my @c3d = ( $self-> light3DColor, $self-> dark3DColor);
 	if ( $self-> enabled) {
 		@clr = ($self-> color, $self-> backColor);
+		$clr[0] = $self->hiliteColor if $skin eq 'flat';
 	} else {
 		@clr = ($self-> disabledColor, $self-> disabledBackColor);
 	}
@@ -169,14 +193,20 @@ sub on_paint
 	my $btx  = $self-> { btx};
 	my $v    = $self-> { vertical};
 	my ( $maxx, $maxy) = ( $size[0]-1, $size[1]-1);
-	$canvas-> color( $c3d[0]);
-	$canvas-> line( 0, 0, $maxx - 1, 0);
-	$canvas-> line( $maxx, 0, $maxx, $maxy - 1);
-	$canvas-> color( $c3d[1]);
-	$canvas-> line( 0, $maxy, $maxx, $maxy);
-	$canvas-> line( 0, 0, 0, $maxy);
-	{
-		$self-> draw_pad( $canvas, b1 => $clr[1]);
+
+	if ( $skin ne 'flat') {
+		$canvas-> color( $c3d[0]);
+		$canvas-> line( 0, 0, $maxx - 1, 0);
+		$canvas-> line( $maxx, 0, $maxx, $maxy - 1);
+		$canvas-> color( $c3d[1]);
+		$canvas-> line( 0, $maxy, $maxx, $maxy);
+		$canvas-> line( 0, 0, 0, $maxy);
+	}
+
+	my $collapsed = ($v ? $maxy : $maxx) < $self->{minThumbSize};
+
+	unless ( $collapsed ) {
+		$self-> draw_pad( $canvas, b1 => $clr[1], $skin);
 		$canvas-> color( $self-> {b1}-> { enabled} ? $clr[ 0] : $self-> disabledColor);
 		my $a = $self-> { b1}-> { pressed} ? 1 : 0;
 		my @spot = map { int($_ + .5) } $v ? (
@@ -192,8 +222,9 @@ sub on_paint
 		$canvas-> fillpoly( [ @spot])
 			if $maxx > 10 && $maxy > 10;
 	}
-	{
-		$self-> draw_pad( $canvas, b2 => $clr[1]);
+
+	unless ( $collapsed ) {
+		$self-> draw_pad( $canvas, b2 => $clr[1], $skin);
 		$canvas-> color( $self-> {b2}-> { enabled} ? $clr[ 0] : $self-> disabledColor);
 		my $a = $self-> { b2}-> { pressed} ? 1 : 0;
 		my @spot = map { int($_ + .5) } $v ? (
@@ -210,6 +241,7 @@ sub on_paint
 			if $maxx > 10 && $maxy > 10;
 		$canvas-> color( $clr[ 1]);
 	}
+
 	if ( $self-> { tab}-> { enabled})
 	{
 		my @rect = @{$self-> { tab}-> { rect}};
@@ -220,28 +252,31 @@ sub on_paint
 			if ( defined $self-> { $_}-> {rect})
 			{
 				my @r = @{$self-> {$_}-> {rect}};
-				$canvas-> color( $c3d[1]);
-				$v ? ( $canvas-> line( $r[0]-1, $r[1], $r[0]-1, $r[3])):
-					( $canvas-> line( $r[0], $r[3]+1, $r[2], $r[3]+1));
+				if ( $skin ne 'flat') {
+					$canvas-> color( $c3d[1]);
+					$v ? ( $canvas-> line( $r[0]-1, $r[1], $r[0]-1, $r[3])):
+						( $canvas-> line( $r[0], $r[3]+1, $r[2], $r[3]+1));
+				}
 				$canvas-> color( $self-> {$_}-> {pressed} ? $clr[0] : $clr[1]);
-				if ( $self->{style} eq 'xp') {
+				if ( $skin eq 'xp') {
 					$canvas-> backColor( $c3d[0]);
+					$canvas-> rop2(rop::CopyPut);
 					$canvas-> fillPattern([(0xAA,0x55) x 4]);
 					$canvas-> bar( @r);
 					$canvas-> fillPattern(fp::Solid);
+					$canvas-> rop2(rop::NoOper);
 				} else {
 					$canvas-> bar( @r);
 				}
 			}
 		}
 
-		$self-> draw_pad( $canvas, tab => $clr[1]);
+		$self-> draw_pad( $canvas, tab => $clr[1], $skin);
 		if (
 			$self-> {minThumbSize} > 8 &&
-			$self->{style} ne 'xp' &&
+			$skin eq 'classic' &&
 			(( $v ? $maxx : $maxy) > 10)
-		)
-		{
+		) {
 			$canvas-> color( $clr[ 0]);
 			$canvas-> backColor( $c3d[ 0]);
 			if ( $v)
@@ -253,9 +288,11 @@ sub on_paint
 				my $stx = int($maxx / 3) + ( $self-> { tab} -> { pressed} ? 1 : 0);
 				my $lnx = int($maxx / 3);
 				$lnx += $maxx - $lnx * 3;
+				$canvas-> rop2(rop::CopyPut);
 				$canvas-> fillPattern([(0xff, 0) x 4]);
 				$canvas-> fillPatternOffset($stx, $sty);
 				$canvas-> bar( $stx, $sty - 1, $stx + $lnx, $sty + 9);
+				$canvas-> rop2(rop::NoOper);
 			} else {
 				my $stx = $rect[0] +
 					int($lenx / 2) -
@@ -264,22 +301,29 @@ sub on_paint
 				my $sty = int($maxy / 3) - ( $self-> { tab} -> { pressed} ? 1 : 0);
 				my $lny = int($maxy / 3);
 				$lny += $maxy - $lny * 3;
+				$canvas-> rop2(rop::CopyPut);
 				$canvas-> fillPattern([(0xAA) x 8]);
 				$canvas-> fillPatternOffset($stx, $sty);
 				$canvas-> bar( $stx - 1, $sty, $stx + 9, $sty + $lny);
+				$canvas-> rop2(rop::NoOper);
 			}
 		}
 	} else {
-		my @r = @{$self-> {groove}-> {rect}};
-		$canvas-> color( $c3d[1]);
-		$v ?
-			( $canvas-> line( $r[0]-1, $r[1], $r[0]-1, $r[3])):
-			( $canvas-> line( $r[0], $r[3]+1, $r[2], $r[3]+1));
+		my @r = $collapsed ? 
+			( 0, 0, $maxx, $maxy ) :
+			@{$self-> {groove}-> {rect}};
+		if ( $skin ne 'flat') {
+			$canvas-> color( $c3d[1]);
+			$v ?
+				( $canvas-> line( $r[0]-1, $r[1], $r[0]-1, $r[3])):
+				( $canvas-> line( $r[0], $r[3]+1, $r[2], $r[3]+1));
+		}
 		$canvas-> color( $clr[1]);
 
-		if ( $self->{style} eq 'xp') {
-			$canvas-> backColor( $c3d[0]);
+		if ( $skin eq 'xp') {
+			$canvas-> backColor( $c3d[1]);
 			$canvas-> fillPattern([(0xAA,0x55) x 4]);
+			$canvas-> rop2(rop::CopyPut);
 			$canvas-> bar( @r);
 			$canvas-> fillPattern(fp::Solid);
 		} else {
@@ -291,6 +335,7 @@ sub on_paint
 sub translate_point
 {
 	my ( $self, $x, $y) = @_;
+	return undef if ($self->vertical ? $self->height : $self->width) < $self->{minThumbSize};
 	for( qw(b1 b2 tab left right groove))
 	{
 		next unless defined $self-> {$_}-> {rect};
@@ -457,6 +502,12 @@ sub on_mouseup
 	}
 }
 
+sub on_mouseenter
+{
+	my $self = shift;
+	$self-> fader_in_mouse_enter if $self->enabled;
+}
+
 sub on_mousemove
 {
 	my ( $self, $mod, $x, $y) = @_;
@@ -516,9 +567,12 @@ sub on_mousemove
 sub on_mouseleave
 {
 	my $self = shift;
-	$self-> repaint if defined( delete $self->{prelight} );
+	return unless $self->enabled;
+	$self-> fader_out_mouse_leave;
 }
 
+sub on_fadeout { delete shift->{prelight} }
+
 sub on_mousewheel
 {
 	my ( $self, $mod, $x, $y, $z) = @_;
@@ -629,6 +683,33 @@ sub reset
 			$self-> { right}-> {rect} = ( $r2[ 0] < $r2[ 2]) ? [ @r2] : undef;
 		}
 	}
+
+	if ($self->skin eq 'flat') {
+		for my $part ( qw(b1 b2 tab left right groove)) {
+			my $r = $self->{$part}->{rect} or next;
+			if ( $v ) {
+				$r->[0]--;
+				$r->[2]++;
+			} else {
+				$r->[1]--;
+				$r->[3]++;
+			}
+		}
+		for my $part ( qw(left right groove)) {
+			my $r = $self->{$part}->{rect} or next;
+			$v ? $r->[0]-- : $r->[3]++;
+		}
+		if ( $v ) {
+			$self->{b1}->{rect}->[3]++;
+			$self->{b2}->{rect}->[1]--;
+			$self->{left}->{rect}->[1]++ if $self->{left}->{rect};
+		} else {
+			$self->{b1}->{rect}->[0]--;
+			$self->{b2}->{rect}->[2]++;
+			$self->{right}->{rect}->[0]++ if $self->{right}->{rect};
+		}
+	}
+
 	$self-> cursorVisible( $self-> { tab}-> { enabled});
 }
 
@@ -714,7 +795,7 @@ sub set_min_thumb_size
 	my ( $self, $mts) = @_;
 	$mts = 1 if $mts < 0;
 	return if $mts == $self-> {minThumbSize};
-	$self-> {minThumbSize} = $mts;
+	$self-> {minThumbSize} = int($mts + .5);
 	$self-> reset;
 	$self-> repaint;
 }
@@ -733,35 +814,44 @@ sub get_default_size
 	return @stdMetrics;
 }
 
-sub value        {($#_)?$_[0]-> set_value       ($_[1])                 : return $_[0]-> {value}       }
+sub skin
+{
+	return $_[0]->SUPER::skin unless $#_;
+	my $self = shift;
+	$self->SUPER::skin($_[1]);
+	$self->reset if defined $self->{value};
+	$self->repaint;
+}
+
 sub autoTrack    {($#_)?$_[0]-> {autoTrack} =    $_[1]                  : return $_[0]-> {autoTrack}   }
+sub max          {($#_)?$_[0]-> set_bounds($_[0]-> {'min'}, $_[1])      : return $_[0]-> {max};}
 sub min          {($#_)?$_[0]-> set_bounds($_[1], $_[0]-> {'max'})      : return $_[0]-> {min};}
 sub minThumbSize {($#_)?$_[0]-> set_min_thumb_size($_[1])               : return $_[0]-> {minThumbSize};}
-sub max          {($#_)?$_[0]-> set_bounds($_[0]-> {'min'}, $_[1])      : return $_[0]-> {max};}
-sub step         {($#_)?$_[0]-> set_steps ($_[1], $_[0]-> {'pageStep'}) : return $_[0]-> {step};}
-sub style        { $#_ ? ($_[0]->{style} = $_[1], $_[0]->repaint) : $_[0]->{style} }
 sub pageStep     {($#_)?$_[0]-> set_steps ($_[0]-> {'step'}, $_[1])     : return $_[0]-> {pageStep};}
 sub partial      {($#_)?$_[0]-> set_proportion ($_[1], $_[0]-> {'whole'}): return $_[0]-> {partial};}
-sub whole        {($#_)?$_[0]-> set_proportion($_[0]-> {'partial'},$_[1]): return $_[0]-> {whole};}
+sub skins        {(shift->SUPER::skins, 'xp')}
+sub step         {($#_)?$_[0]-> set_steps ($_[1], $_[0]-> {'pageStep'}) : return $_[0]-> {step};}
+sub value        {($#_)?$_[0]-> set_value       ($_[1])                 : return $_[0]-> {value}       }
 sub vertical     {($#_)?$_[0]-> set_vertical  ($_[1])                   : return $_[0]-> {vertical}    }
+sub whole        {($#_)?$_[0]-> set_proportion($_[0]-> {'partial'},$_[1]): return $_[0]-> {whole};}
 
 1;
 
+=pod
+
 =head1 NAME
 
-Prima::ScrollBar - standard scroll bars class
+Prima::ScrollBar - scroll bars
 
 =head1 DESCRIPTION
 
-The class C<Prima::ScrollBar> implements both vertical and horizontal
-scrollbars in I<Prima>. This is a purely Perl class without any
-C-implemented parts except those inherited from C<Prima::Widget>.
+C<Prima::ScrollBar> implements standard vertical and horizontal scrollbars
 
 =head1 SYNOPSIS
 
 	use Prima::ScrollBar;
 
-	my $sb = Prima::ScrollBar-> create( owner => $group, %rest_of_profile);
+	my $sb = Prima::ScrollBar->new( owner => $group, %rest_of_profile);
 	my $sb = $group-> insert( 'ScrollBar', %rest_of_profile);
 
 	my $isAutoTrack = $sb-> autoTrack;
@@ -804,23 +894,21 @@ C-implemented parts except those inherited from C<Prima::Widget>.
 
 =item autoTrack BOOLEAN
 
-Tells the widget if it should send
-C<Change> notification during mouse tracking events.
-Generally it should only be set to 0 on very slow computers.
+Tells the widget if it should send the C<Change> notification during mouse
+tracking events.  Generally, it should only be set to 0 on very slow computers.
 
-Default value is 1 (logical true).
+The default value: 1
 
 =item growMode INTEGER
 
-Default value is gm::GrowHiX, i.e. the scrollbar will try
-to maintain the constant distance from its right edge to its
-owner's right edge as the owner changes its size.
-This is useful for horizontal scrollbars.
+The default value is C<gm::GrowHiX>, i.e. the scrollbar will try to maintain
+the constant distance from its right edge to its owner's right edge as the
+owner changes its size.  This is useful for horizontal scrollbars.
 
 =item height INTEGER
 
-Default value is $Prima::ScrollBar::stdMetrics[1], which is an operating
-system dependent value determined with a call to
+The default value is $Prima::ScrollBar::stdMetrics[1], which is an operating
+system-dependent value determined with a call to
 C<Prima::Application-E<gt> get_default_scrollbar_metrics>.  The height is
 affected because by default the horizontal C<ScrollBar> will be
 created.
@@ -829,59 +917,54 @@ created.
 
 Sets the upper limit for C<value>.
 
-Default value: 100.
+The default value: 100.
 
 =item min INTEGER
 
 Sets the lower limit for C<value>.
 
-Default value: 0
+The default value: 0
 
 =item minThumbSize INTEGER
 
 A minimal thumb breadth in pixels. The thumb cannot have
-main dimension lesser than this.
+a main dimension lesser than this.
 
-Default value: 21
+The default value: 21
 
 =item pageStep INTEGER
 
-This determines the increment/decrement to
-C<value> during "page"-related operations, for example clicking the mouse
-on the strip outside the thumb, or pressing C<PgDn> or C<PgUp>.
+This determines the increment/decrement to C<value> during the operations that
+suppose to scroll by pages, for example clicking the mouse on the strip outside
+the thumb, or pressing C<PgDn> or C<PgUp>.
 
-Default value: 10
+The default value: 10
 
 =item partial INTEGER
 
-This tells the scrollbar how many of imaginary
+This tells the scrollbar how many imaginary
 units the thumb should occupy. See C<whole> below.
 
-Default value: 10
+The default value: 10
 
 =item selectable BOOLEAN
 
-Default value is 0 (logical false).  If set to 1 the widget receives keyboard
-focus; when in focus, the thumb is blinking.
+The default value is 0.  If set to 1 the widget receives keyboard
+focus; when in focus, the thumb bar is blinking.
 
 =item step INTEGER
 
 This determines the minimal increment/decrement to C<value> during
 mouse/keyboard interaction.
 
-Default value is 1.
-
-=item style STRING = [ 'xp' | 'os2 ]
-
-Defines the scrollbar drawing style, that was historically different on windows
-and the other platforms. It is possible to define own styles.
+The default value is 1
 
 =item value INTEGER
 
 A basic scrollbar property; reflects the imaginary position between C<min> and
 C<max>, which corresponds directly to the position of the thumb.
 
-Default value is 0.
+The default value is 0
 
 =item vertical BOOLEAN
 
@@ -889,18 +972,17 @@ Determines the main scrollbar style.  Set this to 1 when the scrollbar style is
 vertical, 0 - horizontal. The property can be changed at run-time, so the
 scrollbars can morph from horizontal to vertical and vice versa.
 
-Default value is 0 (logical false).
+The default value is 0
 
 =item whole INTEGER
 
-This tells the scrollbar how many of imaginary units correspond to the whole
+This tells the scrollbar how many imaginary units correspond to the whole
 length of the scrollbar.  This value has nothing in common with C<min> and
-C<max>.  You may think of the combination of C<partial> and C<whole> as of the
+C<max>.  You may think of the combination of C<partial> and C<whole> as the
 proportion between the visible size of something (document, for example) and
-the whole size of that "something".  Useful to struggle against infamous "bird"
-size of Windows scrollbar thumbs.
+the whole size of that "something".
 
-Default value is 100.
+The default value is 100.
 
 =back
 
@@ -910,8 +992,8 @@ Default value is 100.
 
 =item get_default_size
 
-Returns two integers, the default platform dependant width
-of a vertical scrollbar and height of a horizontal scrollbar.
+Returns two integers, the default platform-dependant width
+of a vertical scrollbar and the height of a horizontal scrollbar.
 
 =back
 
@@ -921,16 +1003,15 @@ of a vertical scrollbar and height of a horizontal scrollbar.
 
 =item Change
 
-The C<Change> notification is sent whenever the thumb position of scrollbar is
-changed, subject to a certain limitations when C<autoTrack> is 0. The
-notification conforms the general I<Prima> rule: it is sent when appropriate,
-regardless to whether this was a result of user interaction, or a side effect
-of some method programmer has called.
+The C<Change> notification is sent whenever the thumb position of the scrollbar is
+changed, subject to certain limitations when C<autoTrack> is 0. The
+notification is sent when appropriate, regardless of whether due to the user
+interaction or a side effect of some method the programmer has called.
 
 =item Track
 
-If C<autoTrack> is 0, called when the user changes the thumb position by the
-mouse instead of C<Change>.
+If C<autoTrack> is 0, called when the user changes the thumb position with the
+mouse.
 
 =back
 
@@ -940,7 +1021,7 @@ mouse instead of C<Change>.
 	use Prima::Application name => 'ScrollBar test';
 	use Prima::ScrollBar;
 
-	my $w = Prima::Window-> create(
+	my $w = Prima::Window->new(
 		text => 'ScrollBar test',
 		size => [300,200]);
 
@@ -956,7 +1037,7 @@ mouse instead of C<Change>.
 
 =head1 SEE ALSO
 
-L<Prima>, L<Prima::Widget>, L<Prima::IntUtils>, F<examples/rtc.pl>, F<examples/scrolbar.pl>
+L<Prima>, L<Prima::Widget>, F<examples/scrollbar.pl>, F<examples/scrollbar2.pl>
 
 =head1 AUTHORS
 
