@@ -252,7 +252,7 @@ sub abs_path
 	return undef unless Prima::Utils::chdir($path);
 	my $realpath = Prima::Utils::getcwd();
 	if (! ((_d $cwd) && (Prima::Utils::chdir($cwd)))) {
-		_croak("Cannot chdir back to $cwd: $!");
+		croak("Cannot chdir back to $cwd: $!");
 	}
 
 	return $realpath;
@@ -272,10 +272,10 @@ Since perl win32 unicode support for files is unexistent, Prima has its own
 parallel set of functions mimicking native functions, ie open, chdir etc. This
 means that files with names that cannot be converted to ANSI (ie
 user-preferred) codepage are not visible in perl, but the functions below
-mitigate that problem.
+mitigate this problem.
 
-This module exports the unicode-aware functions from C<Prima::Utils> to override
-the core functions. Read more in L<Prima::Utils/"Unicode-aware filesystem functions">.
+The module exports the unicode-aware functions from C<Prima::Utils> to override
+the core functions. Read more in L<Prima::Utils/"Unicode-aware file system functions">.
 
 =head2 SYNOPSIS
 
