@@ -16,7 +16,7 @@ extern "C" {
 
 
 static Bool
-primitive( Handle self, Bool fill, char * method, ...)
+stroke( Handle self, char * method, ...)
 {
 	Bool r;
 	SV * ret;
@@ -27,7 +27,7 @@ primitive( Handle self, Bool fill, char * method, ...)
 	SAVETMPS;
 	strcpy(format, "<");
 	strncat(format, method, 255);
-	ret = call_perl_indirect( self, fill ? "fill_primitive" : "stroke_primitive", format, true, false, args);
+	ret = call_perl_indirect( self, "stroke_primitive", format, true, false, args);
 	va_end( args);
 	r = ret ? SvTRUE( ret) : false;
 	FREETMPS;
@@ -36,28 +36,6 @@ primitive( Handle self, Bool fill, char * method, ...)
 }
 
 
-Bool
-Drawable_arc( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
-{
-	gpCHECK(false);
-	while ( startAngle > endAngle ) endAngle += 360.0;
-	return primitive( self, 0, "snnnnnn", "arc", x, y, dX, dY, startAngle, endAngle);
-}
-
-Bool
-Drawable_chord( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
-{
-	gpCHECK(false);
-	return primitive( self, 0, "snnnnnn", "chord", x, y, dX, dY, startAngle, endAngle);
-}
-
-Bool
-Drawable_ellipse( Handle self, double x, double y,  double dX, double dY)
-{
-	gpCHECK(false);
-	return primitive( self, 0, "snnnn", "ellipse", x, y, dX, dY);
-}
-
 Bool
 Drawable_bar( Handle self, double x1, double y1, double x2, double y2)
 {
@@ -201,27 +179,6 @@ Drawable_clear( Handle self, double x1, double y1, double x2, double y2)
 	return ok;
 }
 
-Bool
-Drawable_fill_chord( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
-{
-	gpCHECK(false);
-	return primitive( self, 1, "snnnnnn", "chord", x, y, dX, dY, startAngle, endAngle);
-}
-
-Bool
-Drawable_fill_ellipse( Handle self, double x, double y,  double dX, double dY)
-{
-	gpCHECK(false);
-	return primitive( self, 1, "snnnn", "ellipse", x, y, dX, dY);
-}
-
-Bool
-Drawable_fill_sector( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
-{
-	gpCHECK(false);
-	return primitive( self, 1, "snnnnnn", "sector", x, y, dX, dY, startAngle, endAngle);
-}
-
 Bool
 Drawable_fillpoly(Handle self, SV * points)
 {
@@ -285,7 +242,7 @@ Drawable_line(Handle self, double x1, double y1, double x2, double y2)
 {
 	gpCHECK(false);
 	if (EMULATED_LINE)
-		return primitive( self, 0, "snnnn", "line", x1, y1, x2, y2);
+		return stroke( self, "snnnn", "line", x1, y1, x2, y2);
 
 	prima_matrix_apply( VAR_MATRIX, &x1, &y1);
 	prima_matrix_apply( VAR_MATRIX, &x2, &y2);
@@ -337,7 +294,7 @@ Drawable_lines(Handle self, SV * lines)
 	gpCHECK(false);
 
 	if (EMULATED_LINE)
-		return primitive( self, 0, "sS", "lines", lines);
+		return stroke( self, "sS", "lines", lines);
 	else
 		return read_polypoints( self, lines, "Drawable::lines", 2, apc_gp_draw_poly2);
 }
@@ -347,7 +304,7 @@ Drawable_polyline(Handle self, SV * lines)
 {
 	gpCHECK(false);
 	if (EMULATED_LINE)
-		return primitive( self, 0, "sS", "line", lines);
+		return stroke( self, "sS", "line", lines);
 	else
 		return read_polypoints( self, lines, "Drawable::polyline", 2, apc_gp_draw_poly);
 }
@@ -360,7 +317,7 @@ Drawable_rectangle( Handle self, double x1, double y1, double x2, double y2)
 
 	gpCHECK(false);
 	if ( EMULATED_LINE )
-		return primitive( self, 0, "snnnn", "rectangle", x1,y1,x2,y2);
+		return stroke( self, "snnnn", "rectangle", x1,y1,x2,y2);
 
 	if ( prima_matrix_is_square_rectangular( VAR_MATRIX, &nrect, npoly)) {
 		Rect r;
@@ -374,13 +331,6 @@ Drawable_rectangle( Handle self, double x1, double y1, double x2, double y2)
 	}
 }
 
-Bool
-Drawable_sector( Handle self, double x, double y, double dX, double dY, double startAngle, double endAngle)
-{
-	gpCHECK(false);
-	return primitive( self, 0, "snnnnnn", "sector", x, y, dX, dY, startAngle, endAngle);
-}
-
 #ifdef __cplusplus
 }
 #endif
