@@ -173,7 +173,7 @@ Drawable_text_out( Handle self, SV * text, int x, int y, int from, int len)
 		STRLEN dlen;
 		char * c_text = SvPV( text, dlen);
 		Bool   utf8 = prima_is_utf8_sv( text);
-		CHECK_GP(false);
+		gpCHECK(false);
 		if ( utf8) dlen = prima_utf8_length(c_text, dlen);
 		if ((len = Drawable_check_length(from,len,dlen)) == 0)
 			return true;
@@ -183,7 +183,7 @@ Drawable_text_out( Handle self, SV * text, int x, int y, int from, int len)
 		if ( !ok) perl_error();
 	} else if ( SvTYPE( SvRV( text)) == SVt_PVAV) {
 		GlyphsOutRec t;
-		CHECK_GP(false);
+		gpCHECK(false);
 		if (!Drawable_read_glyphs(&t, text, 0, "Drawable::text_out"))
 			return false;
 		if (t.len == 0)
@@ -234,13 +234,13 @@ Drawable_get_glyphs_width( Handle self, PGlyphsOutRec t, Bool add_overhangs)
 int
 Drawable_get_text_width( Handle self, SV * text, int flags, int from, int len)
 {
-	gpARGS;
+	dmARGS;
 	int res;
 
 	if ( !SvROK( text )) {
 		STRLEN dlen;
 		char * c_text = SvPV( text, dlen);
-		CHECK_GP(0);
+		dmCHECK(0);
 		if ( prima_is_utf8_sv( text)) {
 			dlen = utf8_length(( U8*) c_text, ( U8*) c_text + dlen);
 			flags |= toUTF8;
@@ -249,12 +249,12 @@ Drawable_get_text_width( Handle self, SV * text, int flags, int from, int len)
 		if (( len = Drawable_check_length(from,len,dlen)) == 0)
 			return 0;
 		c_text = Drawable_hop_text(c_text, flags & toUTF8, from);
-		gpENTER(0);
+		dmENTER(0);
 		res = apc_gp_get_text_width( self, c_text, len, flags);
-		gpLEAVE;
+		dmLEAVE;
 	} else if ( SvTYPE( SvRV( text)) == SVt_PVAV) {
 		GlyphsOutRec t;
-		CHECK_GP(0);
+		dmCHECK(0);
 		if (!Drawable_read_glyphs(&t, text, 0, "Drawable::get_text_width"))
 			return false;
 		if (t.len == 0)
@@ -264,35 +264,51 @@ Drawable_get_text_width( Handle self, SV * text, int flags, int from, int len)
 		Drawable_hop_glyphs(&t, from, len);
 		if (t.advances)
 			return Drawable_get_glyphs_width(self, &t, flags & toAddOverhangs);
-		gpENTER(0);
+		dmENTER(0);
 		res = apc_gp_get_glyphs_width( self, &t);
-		gpLEAVE;
+		dmLEAVE;
 	} else {
 		SV * ret;
-		gpENTER(0);
+		dmENTER(0);
 		ret = sv_call_perl(text, "get_text_width", "<Hiii", self, flags, from, len);
-		gpLEAVE;
+		dmLEAVE;
 		res = (ret && SvOK(ret)) ? SvIV(ret) : 0;
 	}
 
 	return res;
 }
 
-static void
-get_glyphs_box( Handle self, PGlyphsOutRec t, Point * pt)
+NPoint
+Drawable_trig_cache( Handle self )
 {
-	Bool text_out_baseline;
+	if ( !is_opt(optFontTrigCache)) {
+		if ( var-> font. direction == 0.0 ) {
+			var->trig_cache.x = 0.0;
+			var->trig_cache.y = 1.0;
+		} else {
+#define GRAD 57.29577951
+			var->trig_cache.x = sin( var-> font. direction / GRAD);
+			var->trig_cache.y = cos( var-> font. direction / GRAD);
+#undef GRAD
+		}
+		opt_set(optFontTrigCache);
+	}
+	return var->trig_cache;
+}
 
-	t-> flags = 0;
+void
+Drawable_calculate_text_box( Handle self, int advance, Bool text_out_baseline, Point ovx, Point * pt)
+{
+	if ( ovx.x > 0 ) ovx.x = 0;
+	if ( ovx.y > 0 ) ovx.y = 0;
 
 	pt[0].y = pt[2]. y = var-> font. ascent - 1;
 	pt[1].y = pt[3]. y = - var-> font. descent;
-	pt[4].y = pt[0]. x = pt[1].x = 0;
-	pt[3].x = pt[2]. x = pt[4].x = Drawable_get_glyphs_width(self, t, false);
+	pt[4].y = 0;
+	pt[4].x = advance;
+	pt[3].x = pt[2]. x = advance - ovx.y;
+	pt[0]. x = pt[1].x = ovx.x;
 
-	text_out_baseline = ( my-> textOutBaseline == Drawable_textOutBaseline) ?
-		apc_gp_get_text_out_baseline(self) :
-		my-> get_textOutBaseline(self);
 	if ( !text_out_baseline ) {
 		int i = 4, d = var->font. descent;
 		while ( i--) pt[i]. y += d;
@@ -300,30 +316,69 @@ get_glyphs_box( Handle self, PGlyphsOutRec t, Point * pt)
 
 	if ( var-> font. direction != 0) {
 		int i;
-#define GRAD 57.29577951
-		float s = sin( var-> font. direction / GRAD);
-		float c = cos( var-> font. direction / GRAD);
-#undef GRAD
+		NPoint c = my->trig_cache(self);
 		for ( i = 0; i < 5; i++) {
-			float x = pt[i]. x * c - pt[i]. y * s;
-			float y = pt[i]. x * s + pt[i]. y * c;
-			pt[i]. x = x + (( x > 0) ? 0.5 : -0.5);
-			pt[i]. y = y + (( y > 0) ? 0.5 : -0.5);
+			float x = pt[i]. x * c.y - pt[i]. y * c.x;
+			float y = pt[i]. x * c.x + pt[i]. y * c.y;
+			pt[i].x = x + (( x > 0) ? 0.5 : -0.5);
+			pt[i].y = y + (( y > 0) ? 0.5 : -0.5);
 		}
 	}
 }
 
+void
+Drawable_get_glyphs_box( Handle self, PGlyphsOutRec t, Point * pt)
+{
+	int l;
+	Bool text_out_baseline;
+	Point ovx = {0,0};
+	SaveFont savefont;
+	FontABC *abc;
+
+	t-> flags = 0;
+	text_out_baseline = ( my-> textOutBaseline == Drawable_textOutBaseline) ?
+		apc_gp_get_text_out_baseline(self) :
+		my-> get_textOutBaseline(self);
+	l = Drawable_get_glyphs_width(self, t, false);
+
+	if ( t->fonts )
+		Drawable_save_font(self, &savefont);
+
+	if ( t-> fonts )
+		Drawable_switch_font( self, &savefont, t-> fonts[0]);
+
+	if (( abc = Drawable_call_get_font_abc( self, t->glyphs[0], t->glyphs[0], toGlyphs)) != NULL) {
+		int l = t->len - 1;
+		ovx.x = abc->a;
+		if ( t->glyphs[0] == t->glyphs[l] && ( !t->fonts || t->fonts[0] == t->fonts[l])) {
+			ovx.y = abc->c;
+			free(abc);
+		} else {
+			free(abc);
+			if (( abc = Drawable_call_get_font_abc( self, t->glyphs[l], t->glyphs[l], toGlyphs)) != NULL) {
+				ovx.y = abc->c;
+				free(abc);
+			}
+		}
+	}
+
+	if ( t->fonts )
+		Drawable_restore_font(self, &savefont);
+
+	Drawable_calculate_text_box(self, l, text_out_baseline, ovx, pt);
+}
+
 SV *
 Drawable_get_text_box( Handle self, SV * text, int from, int len )
 {
-	gpARGS;
+	dmARGS;
 	Point * p;
 	AV * av;
 	int i, flags = 0;
 	if ( !SvROK( text )) {
 		STRLEN dlen;
 		char * c_text = SvPV( text, dlen);
-		CHECK_GP(NULL_SV);
+		dmCHECK(NULL_SV);
 
 		if ( prima_is_utf8_sv( text)) {
 			dlen = utf8_length(( U8*) c_text, ( U8*) c_text + dlen);
@@ -332,12 +387,12 @@ Drawable_get_text_box( Handle self, SV * text, int from, int len )
 		if ((len = Drawable_check_length(from,len,dlen)) == 0)
 			return newRV_noinc(( SV *) newAV());
 		c_text = Drawable_hop_text(c_text, flags & toUTF8, from);
-		gpENTER( newRV_noinc(( SV *) newAV()));
+		dmENTER( newRV_noinc(( SV *) newAV()));
 		p = apc_gp_get_text_box( self, c_text, len, flags);
-		gpLEAVE;
+		dmLEAVE;
 	} else if ( SvTYPE( SvRV( text)) == SVt_PVAV) {
 		GlyphsOutRec t;
-		CHECK_GP(NULL_SV);
+		dmCHECK(NULL_SV);
 		if (!Drawable_read_glyphs(&t, text, 0, "Drawable::get_text_box"))
 			return false;
 		if (( len = Drawable_check_length(from,len,t.len)) == 0)
@@ -346,17 +401,17 @@ Drawable_get_text_box( Handle self, SV * text, int from, int len )
 		if (t.advances) {
 			if (!( p = malloc( sizeof(Point) * 5 )))
 				return newRV_noinc(( SV *) newAV());
-			get_glyphs_box(self, &t, p);
+			Drawable_get_glyphs_box(self, &t, p);
 		} else {
-			gpENTER( newRV_noinc(( SV *) newAV()));
+			dmENTER( newRV_noinc(( SV *) newAV()));
 			p = apc_gp_get_glyphs_box( self, &t);
-			gpLEAVE;
+			dmLEAVE;
 		}
 	} else {
 		SV * ret;
-		gpENTER( newRV_noinc(( SV *) newAV()));
+		dmENTER( newRV_noinc(( SV *) newAV()));
 		ret = newSVsv(sv_call_perl(text, "get_text_box", "<Hii", self, from, len ));
-		gpLEAVE;
+		dmLEAVE;
 		return ret;
 	}
 
