@@ -49,11 +49,11 @@ Drawable_call_get_font_abc( Handle self, unsigned int from, unsigned int to, int
 		abc = apc_gp_get_font_abc( self, from, to, flags);
 		if ( !abc) return NULL;
 	} else if ( my-> get_font_abc == Drawable_get_font_abc) {
-		gpARGS;
-		CHECK_GP(NULL);
-		gpENTER(NULL);
+		dmARGS;
+		dmCHECK(NULL);
+		dmENTER(NULL);
 		abc = apc_gp_get_font_abc( self, from, to, flags);
-		gpLEAVE;
+		dmLEAVE;
 		if ( !abc) return NULL;
 	} else {
 		SV * sv;
@@ -147,7 +147,7 @@ static Bool
 fill_font_ranges( Handle self )
 {
 	if ( Drawable_get_font_ranges == my->get_font_ranges ) {
-		CHECK_GP(false);
+		dmCHECK(false);
 		if ( !var-> font_abc_glyphs_ranges ) {
 			if ( !( var-> font_abc_glyphs_ranges = apc_gp_get_font_ranges(self, &var->font_abc_glyphs_n_ranges)))
 				return false;
@@ -196,10 +196,11 @@ query_abc_range_glyphs( Handle self, GlyphWrapRec * t, unsigned int base)
 		/* different fonts case */
 		Byte * fa;
 		PassiveFontEntry *pfe;
-		int i, font_changed = 0;
+		int i;
 		uint32_t from, to;
 		unsigned int page;
 		char * key;
+		SaveFont savefont;
 		Byte used_fonts[MAX_CHARACTERS / 8], filled_entries[256 / 8];
 
 		from = base * 256;
@@ -210,6 +211,7 @@ query_abc_range_glyphs( Handle self, GlyphWrapRec * t, unsigned int base)
 		used_fonts[0] = 0x01; /* fid = 0 */
 		key = Drawable_font_key(var->font.name, var->font.style);
 		i = PTR2IV(hash_fetch(font_substitutions, key, strlen(key)));
+		Drawable_save_font(self, &savefont);
 		if ( i > 0 ) {
 			/* copy ranges from subst table */
 			pfe = PASSIVE_FONT(i);
@@ -248,9 +250,8 @@ query_abc_range_glyphs( Handle self, GlyphWrapRec * t, unsigned int base)
 			used_fonts[fid >> 3] |= 1 << (fid & 7);
 
 			pfe = PASSIVE_FONT(fid);
-			if ( !Drawable_switch_font(self, fid))
+			if ( !Drawable_switch_font(self, &savefont, fid))
 				continue;
-			font_changed = 1;
 
 			if ( !pfe-> ranges_queried )
 				Drawable_query_ranges(pfe);
@@ -269,13 +270,10 @@ query_abc_range_glyphs( Handle self, GlyphWrapRec * t, unsigned int base)
 				filled_entries[(uv - from) >> 3] |= 1 << ((uv - from) & 7);
 				abc[uv - from] = abc2[uv - from];
 			}
+
+			free(abc2);
 		}
-		if ( font_changed ) {
-			if ( Drawable_set_font == my->set_font && is_opt(optSystemDrawable))
-				apc_gp_set_font( self, &var->font);
-			else
-				my->set_font(self, var->font);
-		}
+		Drawable_restore_font( self, &savefont );
 	}
 NO_FONT_ABC:
 
@@ -320,7 +318,7 @@ fill_tilde_properties(Handle self, TextWrapRec * t, int tildeIndex, int tildePos
 	t-> t_pos  = tildeCharPos + (( t->options & twCollapseTilde ) ? 0 : 1);
 	t-> t_char = t-> text + tildeIndex + 1;
 	if ( t-> utf8_text) {
-		STRLEN len;
+		unsigned int len;
 		uv = prima_utf8_uvchr_end(t-> t_char, t->text + t-> textLen, &len);
 		if ( len == 0 ) return;
 	} else
@@ -341,7 +339,7 @@ fill_tilde_properties(Handle self, TextWrapRec * t, int tildeIndex, int tildePos
 static void
 text_get_libthai_breaks( TextWrapRec* t)
 {
-	STRLEN charlen;
+	unsigned int charlen;
 	char * utf8, thbuf[1024 * sizeof(thwchar_t)];
 	int target_len_xchars, src_len_bytes, got_thai_chars = 0;
 	thwchar_t *u32;
@@ -730,7 +728,7 @@ glyphout2sv(Handle self, int * c, GlyphsOutRec *g, TextWrapRec *tw, GlyphWrapRec
 			k++, l++
 		) {
 			uint32_t uv;
-			STRLEN len;
+			unsigned int len;
 			if (tw->utf8_text) {
 				uv = prima_utf8_uvchr_end(text, tw->text + tw-> textLen, &len);
 				if ( len < 1 ) break;
@@ -888,7 +886,7 @@ Drawable_do_text_wrap( Handle self, TextWrapRec * tw, GlyphWrapRec * gw, uint16_
 	while ( wr.curr.p < wr.limit ) {
 		float dw, c;
 		unsigned int j, nc, ng, wmul = 1;
-		STRLEN len = 1;
+		unsigned int len = 1;
 		uint32_t uv, uv0, last_uv = 0;
 		uint16_t index;
 
@@ -1119,7 +1117,7 @@ Drawable_do_text_wrap( Handle self, TextWrapRec * tw, GlyphWrapRec * gw, uint16_
 static SV*
 string_wrap( Handle self,SV * text, int width, int options, int tabIndent, int from, int len)
 {
-	gpARGS;
+	dmARGS;
 	TextWrapRec t;
 	int * c;
 	SV *ret;
@@ -1131,9 +1129,9 @@ string_wrap( Handle self,SV * text, int width, int options, int tabIndent, int f
 	}
 
 	text_init_wrap_rec( self, text, width, options, tabIndent, from, len, &t);
-	gpENTER(NULL_SV);
+	dmENTER(NULL_SV);
 	c = my->do_text_wrap( self, &t, NULL, NULL);
-	gpLEAVE;
+	dmLEAVE;
 	t.t_pos += from;
 
 	if (( t. options & twReturnFirstLineLength) == twReturnFirstLineLength)
@@ -1161,7 +1159,7 @@ string_wrap( Handle self,SV * text, int width, int options, int tabIndent, int f
 static SV*
 glyphs_wrap( Handle self, SV * text, int width, int options, int from, int len)
 {
-	gpARGS;
+	dmARGS;
 	GlyphWrapRec t;
 	int * c;
 	GlyphsOutRec g;
@@ -1178,9 +1176,9 @@ glyphs_wrap( Handle self, SV * text, int width, int options, int from, int len)
 	if (options & (twExpandTabs|twCollapseTilde|twCalcMnemonic|twCalcTabs|twWordBreak))
 		warn("Drawable::text_wrap(glyphs) does not accept tw::ExpandTabs,tw::CollapseTilde,tw::CalcMnemonic,tw::CalcTabs,tw::WordBreak");
 
-	gpENTER(NULL_SV);
+	dmENTER(NULL_SV);
 	c = my->do_text_wrap( self, NULL, &t, NULL);
-	gpLEAVE;
+	dmLEAVE;
 
 	if (( t. options & twReturnFirstLineLength) == twReturnFirstLineLength)
 		ret = first_line2sv(c, t.count);
@@ -1201,7 +1199,7 @@ glyphs_wrap( Handle self, SV * text, int width, int options, int from, int len)
 static SV*
 string_glyphs_wrap( Handle self, SV * text, int width, int options, int tabIndent, int from, int len, SV * glyphs)
 {
-	gpARGS;
+	dmARGS;
 	SV *qt, *ret, *av = NULL;
 	GlyphsOutRec g;
 	TextWrapRec tw;
@@ -1232,20 +1230,20 @@ string_glyphs_wrap( Handle self, SV * text, int width, int options, int tabInden
 			return qt;
 	}
 
-	gpENTER(NULL_SV);
+	dmENTER(NULL_SV);
 
 	glyph_init_wrap_rec( self, width, options, from, &g, &gw);
 	if ( g.indexes ) {
 		/* log2vis needs to address the whole string */
 		if ( !( log2vis = fill_log2vis(&g, from))) {
-			gpLEAVE;
+			dmLEAVE;
 			warn("not enough memory");
 			return NULL_SV;
 		}
 	}
 
 	c = my->do_text_wrap( self, &tw, &gw, log2vis + from);
-	gpLEAVE;
+	dmLEAVE;
 	tw.t_pos += from;
 
 	if (( options & twReturnFirstLineLength) == twReturnFirstLineLength) {
@@ -1288,13 +1286,13 @@ Drawable_text_wrap( Handle self, SV * text, int width, int options, int tabInden
 		return glyphs_wrap(self, text, width, options, from, len);
 	} else {
 		SV * ret;
-		gpARGS;
-		gpENTER(
+		dmARGS;
+		dmENTER(
 			(( options & twReturnFirstLineLength) == twReturnFirstLineLength) ?
 				newSViv(0) : newRV_noinc(( SV *) newAV())
 		);
 		ret = newSVsv(sv_call_perl(text, "text_wrap", "<Hiiiii", self, width, options, tabIndent, from, len));
-		gpLEAVE;
+		dmLEAVE;
 		return ret;
 	}
 }
