@@ -1085,6 +1085,30 @@ Icon_matrix_transform( Handle self, Matrix matrix, ColorPixel fill, Point *apert
 	return ok;
 }
 
+int
+Icon_get_effective_rop( Handle self, int rop)
+{
+	if ( rop == ropDefault )
+		rop = (var->maskType == 1) ? ropCopyPut : ropBlend;
+	return inherited get_effective_rop(self, rop);
+}
+
+void
+Icon_begin_preserve_type( Handle self, PImagePreserveTypeRec save)
+{
+	inherited begin_preserve_type(self, save);
+	save->mask   = var->maskType;
+
+}
+
+void
+Icon_end_preserve_type( Handle self, PImagePreserveTypeRec save)
+{
+	inherited end_preserve_type(self, save);
+	if ( save->enabled && var->maskType != save->mask)
+		my->set_maskType( self, save->mask);
+}
+
 #ifdef __cplusplus
 }
 #endif
