@@ -124,10 +124,21 @@ Image_handle_event( Handle self, PEvent event)
 void
 Image_done( Handle self)
 {
+	if ( var-> loading_session ) {
+		apc_img_close_load((PImgLoadFileInstance) var-> loading_session);
+		var-> loading_session = NULL;
+	}
+	if ( var-> saving_session ) {
+		apc_img_close_save((PImgSaveFileInstance) var-> saving_session, false);
+		var-> saving_session = NULL;
+	}
+
 	if ( var-> regionData ) {
 		free(var->regionData);
 		var->regionData = NULL;
 	}
+	if ( is_opt(optInFontQuery))
+		my-> end_font_query(self);
 	if ( var->fillPatternImage ) {
 		unprotect_object(var-> fillPatternImage);
 		var->fillPatternImage = NULL_HANDLE;
@@ -137,6 +148,25 @@ Image_done( Handle self)
 	inherited done( self);
 }
 
+Bool
+Image_assert_drawing_mode( Handle self, int mode )
+{
+	switch (mode) {
+	case admStatus:
+		if ( opt_InPaint )
+			return inherited assert_drawing_mode(self, mode);
+		return is_opt(optInFontQuery);
+	case admEnter:
+		return my->begin_font_query(self);
+	case admLeave:
+		/* end_font_query() is okay to linger */
+		return 1;
+	case admAllowed:
+		return 1;
+	}
+	return 0;
+}
+
 Point
 Image_resolution( Handle self, Bool set, Point resolution)
 {
@@ -285,6 +315,8 @@ Image_begin_paint( Handle self)
 		free(var->regionData);
 		var->regionData = NULL;
 	}
+	if ( is_opt(optInFontQuery))
+		my-> end_font_query(self);
 	if ( !inherited begin_paint( self))
 		return false;
 	if ( !( ok = apc_image_begin_paint( self))) {
@@ -305,6 +337,8 @@ Image_begin_paint_info( Handle self)
 		free(var->regionData);
 		var->regionData = NULL;
 	}
+	if ( is_opt(optInFontQuery))
+		my-> end_font_query(self);
 	if ( !inherited begin_paint_info( self))
 		return false;
 	if ( !( ok = apc_image_begin_paint_info( self))) {
@@ -319,12 +353,13 @@ Image_begin_paint_info( Handle self)
 void
 Image_end_paint( Handle self)
 {
-	int oldType = var->type;
+	ImagePreserveTypeRec p;
 	if ( !is_opt( optInDraw)) return;
+	my-> begin_preserve_type(self, &p);
 	apc_image_end_paint( self);
 	inherited end_paint( self);
-	if ( is_opt( optPreserveType) && var->type != oldType) {
-		my->reset( self, oldType, NULL, 0);
+	if ( is_opt( optPreserveType)) {
+		my-> end_preserve_type(self, &p);
 		return;
 	}
 
@@ -555,7 +590,7 @@ Image_put_image_indirect( Handle self, Handle image, int x, int y, int xFrom, in
 
 	if ( is_opt( optInDrawInfo)) return false;
 	if ( image == NULL_HANDLE) return false;
-	if ( is_opt( optInDraw))
+	if ( is_opt( optInDraw)) 
 		return inherited put_image_indirect( self, image, x, y, xFrom, yFrom, xDestLen, yDestLen, xLen, yLen, rop);
 	if ( !kind_of( image, CImage)) return false;
 
@@ -564,6 +599,7 @@ Image_put_image_indirect( Handle self, Handle image, int x, int y, int xFrom, in
 		Image_color2pixel( self, my->get_color(self), colorbuf );
 		color = colorbuf;
 	}
+	rop = CDrawable(image)->get_effective_rop(image, rop);
 
 	prima_matrix_apply_int_to_int(*matrix, &x, &y);
 	if ( prima_matrix_is_translated_only(*matrix)) {
@@ -711,7 +747,7 @@ Image_rop( Handle self, Bool set, int rop)
 	if (!set) return var-> extraROP;
 	if ( rop < 0 ) rop = 0;
 	var-> extraROP = rop;
-	if ( rop > ropNoOper ) rop = ropNoOper;
+	if ( rop > ropWhiteness ) rop = ropCopyPut;
 	apc_gp_set_rop( self, rop);
 	return var-> extraROP;
 }
