@@ -8,14 +8,14 @@ examples/buttons2.pl - Prima button widgets
 
 Demonstrates the variety of built-in buttons functionality.
 Note the "Bits for toolbar" button, which copies and
-pastes its image into the clipboard.
+pastes its image in the clipboard.
 
 =cut
 
 use strict;
 use warnings;
 
-use Prima qw(Buttons StdBitmap), Application => { name => 'Buttons sample' };
+use Prima qw(Buttons StdBitmap Application);
 
 my $w = Prima::MainWindow-> create(
 	text=> "Handmade buttons",
