@@ -1,3 +1,16 @@
+=pod
+
+=head1 NAME
+
+examples/layered.pl - alpha channel demo
+
+=head1 FEATURES
+
+Tests various aspects of alpha-blending in both layered and non-layered
+widgets. 
+
+=cut
+
 use strict;
 use warnings;
 use Prima qw(Application Buttons);
@@ -7,11 +20,9 @@ my $j = Prima::Image->new(
 	height => 100,
 	type => im::Byte,
 );
-$j->begin_paint;
 $j->backColor(cl::Clear);
 $j->clear;
-$j->gradient_ellipse(50, 50, 100, 100, {palette => [cl::Black, cl::White]});
-$j->end_paint;
+$j->new_gradient(palette => [cl::Black, cl::White])->ellipse(50, 50, 100, 100);
 $j->type(im::Byte);
 
 sub icon
@@ -22,14 +33,12 @@ sub icon
 		height => 100,
 		type => im::RGB,
 	);
-	$i->begin_paint;
 	$i->backColor(cl::Clear);
 	$i->clear;
-	$i->gradient_ellipse(50, 50, 100, 100, {
+	$i->new_gradient(
 		palette => [cl::Black, $color],
 		spline  => [0.2,0.8],
-	});
-	$i->end_paint;
+	)-> ellipse( 50, 50, 100, 100 );
 
 	my $k = Prima::Icon->new;
 	$k->combine($i, $j);
@@ -52,16 +61,6 @@ my ( $r, $g, $b ) = map { icon($_) } (cl::LightRed, cl::LightGreen, cl::LightBlu
 my $angle = 0;
 my $pi = 3.14159;
 
-my $i = Prima::Image-> new(
-	width  => 96 * $::application-> font-> width  / 7,  # poor man's calculations of
-	height => 36 * $::application-> font-> height / 16, # button size
-	type => im::BW,
-);
-$i->begin_paint;
-$i->clear;
-$i->fill_ellipse(10,10,10,10);
-$i->end_paint;
-
 my $w = Prima::MainWindow->new(
 	layered  => 1,
 	text     => 'ARGB example',
@@ -92,8 +91,8 @@ my $w = Prima::MainWindow->new(
 		for my $i ( $r, $g, $b ) {
 			my $sin = sin($a) * $r0;
 			my $cos = cos($a) * $r0;
-			my $xx = $cos - $sin; 
-			my $yy = $sin + $cos; 
+			my $xx = $cos - $sin;
+			my $yy = $sin + $cos;
 			$xx -= $x / 2;
 			$yy -= $y / 2;
 			$self->stretch_image( $xx, $yy, $x, $y, $i);
@@ -102,14 +101,16 @@ my $w = Prima::MainWindow->new(
 	}
 );
 
-my $btn = $w-> insert( Button => 
+die "Cannot create a layered window, exiting\n" unless $w->is_surface_layered;
+
+my $btn = $w-> insert( Button =>
 	origin  => [10,10],
-	shape   => $i,
+	autoShaping => 1,
 	text    => '~Quit',
 	onClick => sub { $::application-> close },
 );
 
-$btn->insert( Widget => 
+$btn->insert( Widget =>
 	origin => [10,10],
 	size   => [10,10],
 	onPaint => sub {
@@ -162,8 +163,8 @@ my $widget = Prima::Widget->new(
 		for my $i ( $r, $g, $b ) {
 			my $sin = sin($a) * $r0;
 			my $cos = cos($a) * $r0;
-			my $xx = $cos - $sin; 
-			my $yy = $sin + $cos; 
+			my $xx = $cos - $sin;
+			my $yy = $sin + $cos;
 			$xx -= $x / 2;
 			$yy -= $y / 2;
 			$self->put_image( $xx, $yy, $i);
@@ -174,7 +175,7 @@ my $widget = Prima::Widget->new(
 
 my ($mx, $my) = (1, 1);
 
-$w->insert( Timer => 
+$w->insert( Timer =>
 	timeout => 100,
 	onTick => sub {
 		$angle += 0.1;
