@@ -1,16 +1,28 @@
+=pod
+
+=head1 NAME
+
+examples/lineplot.pl - line plotting API
+
+=head1 FEATURES
+
+Demonstrates Prima line joins, patterns, endings etc.
+
+=cut
+
 use strict;
 use warnings;
 use Prima qw(Application MsgBox);
 
 my %opt = (
-	le  => le::Round,
-	lep => 'le::Round',
+	le  => [le::Round, undef, undef, undef],
 	lj  => lj::Round,
 	ljp => 'lj::Round',
 	lp  => lp::DotDot,
 	lpp => 'lp::DotDot',
 	ml  => 10,
 	lw  => 28,
+	sc  => 1,
 );
 $opt{ml} = 10;
 my $aperture = 12;
@@ -19,6 +31,7 @@ my $prelight;
 my @points;
 @points = (100,200,200,200,200,100,50,10);
 
+
 sub group
 {
 	my $class = shift;
@@ -34,6 +47,27 @@ sub group
 	return @items;
 }
 
+sub legroup
+{
+	my $id = shift;
+	my @items = qw(Flat Square Round Arrow Cusp InvCusp Knob Rect RoundRect Spearhead Tail);
+	@items = map {[
+		"le\:\:$_" => "~$_" => sub {
+			$opt{le}->[$id] = eval $_[1];
+			$_[0]->repaint;
+		}
+	]} @items;
+	unshift @items, [
+		'*' => "~Default" => sub {
+			$opt{le}->[$id] = undef;
+			$_[0]->repaint;
+		}
+	] if $id;
+	$items[0][0]  = "(" . $items[0][0]; 
+	$items[-1][0] = ")" . $items[-1][0]; 
+	return @items;
+}
+
 my $mw;
 
 sub lw
@@ -42,6 +76,12 @@ sub lw
 	$mw->repaint;
 }
 
+sub scale
+{
+	$opt{sc} = $mw->menu->text($_[1]);
+	$mw->repaint;
+}
+
 $mw = Prima::MainWindow->new(
 	size => [ 400, 431 ],
 	text => 'Line plotter',
@@ -51,10 +91,24 @@ $mw = Prima::MainWindow->new(
 			[ '@closed'    => '~Closed'    => sub{shift->repaint} ],
 			[ '@*hairline' => '~Hairline'  => sub{shift->repaint} ],
 			[ '@aa'        => '~Antialias' => sub{shift->repaint} ],
+			[ '@curve'     => 'C~urve'     => sub{shift->repaint} ],
 			[],
 			[ 'E~xit' => sub { $_[0]->destroy } ],
 		]],
-		[ '~End' => [ group le => qw(Flat Square Round) ]],
+		[ '~Ends' => [
+			[ '~Line head'  => [ legroup 0 ]],
+			[ 'Line ~tail'  => [ legroup 1 ]],
+			[ 'Arrow ~head' => [ legroup 2 ]],
+			[ '~Arrow tail' => [ legroup 3 ]],
+		]],
+		[ 'End ~scale' => [
+			[ '(' , 0.5  => \&scale ],
+			[ ''  , 0.75 => \&scale ],
+			[ '*' , 1    => \&scale ],
+			[ ''  , 1.25 => \&scale ],
+			[ ''  , 1.5  => \&scale ],
+			[ ')' , 1.75 => \&scale ],
+		]],
 		[ '~Join' => [
 			group(lj => qw(Round Bevel Miter)),
 			[],
@@ -89,7 +143,12 @@ $mw = Prima::MainWindow->new(
 		my ( $self, $canvas ) = @_;
 		my ( $w, $h ) = $self-> size;
 		$canvas->clear();
-		$canvas->lineEnd($opt{le});
+		$canvas->lineEnd([
+			map {
+				defined ? le::scale( $_, $opt{sc} ) : undef
+			}
+			@{ $opt{le} }
+		]);
 		$canvas->lineJoin($opt{lj});
 		$canvas->miterLimit($opt{ml});
 		my $c = $prelight // $capture // -1;
@@ -99,6 +158,11 @@ $mw = Prima::MainWindow->new(
 			((@points < 6) ? ( 100, 100, @points, 100, 100 ) : @points[0..$#points,0,1]):
 			( 30, 30, @points, $w - 30, $h - 30);
 
+		if ( $self-> menu->checked('curve')) {
+			@xpoints = @{ $canvas->render_spline(\@xpoints) }
+		}
+
+
 		$canvas->color(cl::LightRed);
 		$canvas->lineWidth($opt{lw});
 		$canvas->linePattern($opt{lp});
@@ -195,6 +259,6 @@ $mw = Prima::MainWindow->new(
 	},
 );
 
-$mw->menu->check($opt{$_}) for qw(lep ljp lpp);
+$mw->menu->check($opt{$_}) for qw(ljp lpp);
 
 run Prima;
