@@ -1,16 +1,16 @@
-=pod 
+=pod
 
 =head1 NAME
 
-examples/scrollbar.pl - A Prima toolkit example
+examples/scrollbar.pl - scrollbar example
 
 =head1 FEATURES
 
-Tests correct representation of a color created by a combination
-of R,G and B components. The Prima::Widget::sizeMin and Prima::Widget::sizeMax
-implementation is tested - the Area widget changes its height on a mouse click,
-up to the left button and down to the right button. 
-Note how Area widget maintains its maximum size when the window gets maximized.
+Tests correct representation of colors created by combinations of R,G, and B
+components. The Prima::Widget::sizeMin and Prima::Widget::sizeMax
+implementation is tested too - the Area widget changes its height on a mouse
+click.  Note how the Area widget maintains its maximum size when the window
+gets maximized.
 
 =cut
 
@@ -45,6 +45,7 @@ my $w = MyWindow-> create(
 	bottom  => 100,
 	width   => 300,
 	height  => 300,
+	designScale => [7, 16],
 	borderStyle => bs::Sizeable,
 );
 
@@ -54,7 +55,7 @@ $w-> insert(
 	 origin   => [ 2, 20],
 	 vertical => 0,
 	 max      => 255,
-	 width    => $w-> width - 2,
+	 width    => 298,
 	 onChange => sub { $w-> updateArea },
 );
 $w-> insert(
@@ -63,7 +64,7 @@ $w-> insert(
 	 origin   => [ 2, 40],
 	 vertical => 0,
 	 max      => 255,
-	 width    => $w-> width - 2,
+	 width    => 298,
 	 onChange => sub { $w-> updateArea },
 );
 $w-> insert(
@@ -72,7 +73,7 @@ $w-> insert(
 	 origin   => [ 2, 60],
 	 vertical => 0,
 	 max      => 255,
-	 width    => $w-> width - 2,
+	 width    => 298,
 	 onChange => sub { $w-> updateArea },
 );
 
@@ -82,7 +83,7 @@ $w-> insert(
 	 origin   => [ 2, 80],
 	 vertical => 1,
 	 max      => 255,
-	 height    => $w-> height - 80,
+	 height    => 218,
 );
 
 
