@@ -2,16 +2,14 @@
 
 =head1 NAME
 
-examples/spline.pl - A spline demo
+examples/spline.pl - spline rendering demo
 
 =head1 FEATURES
 
 Demonstrates parabolic splines as graphic primitives.
-The points arount the filled shapes are spline vertexes,
+The points arount the filled shapes are the spline vertexes
 which are dragable by the mouse.
 
-Using L<Prima::Drawable::CurvedText>.
-
 =cut
 
 use strict;
@@ -20,11 +18,11 @@ use Prima qw(Application Drawable::CurvedText);
 
 my @bounds;
 my @points = (
-	10, 10, 
-	130, 30, 
-	20, 20, 
-	20, 300, 
-	50, 300, 
+	10, 10,
+	130, 30,
+	20, 20,
+	20, 300,
+	50, 300,
 );
 my @vectors = (
 	2, 2,
@@ -35,12 +33,12 @@ my @vectors = (
 );
 my @rgb = (
 	0,
-	0, 
+	0,
 	0,
 );
 my @rgb_vectors = (
 	-1.04,
-	1.79, 
+	1.79,
 	1.38,
 );
 my $capture;
@@ -48,6 +46,7 @@ my $aperture = 6 * $::application-> uiScaling;
 
 my $w = Prima::MainWindow-> create(
 	text => 'Spline demo',
+	antialias => (( $ARGV[0] // '' ) eq 'aa'),
 	backColor => cl::Black,
 	buffered => 1,
 	onPaint => sub {
@@ -56,9 +55,11 @@ my $w = Prima::MainWindow-> create(
 
 		my $spline = $canvas-> render_spline( [ @points, @points[0,1]]);
 		$canvas-> fillpoly( $spline);
+
+		$canvas->antialias(0);
 		if ( defined $capture) {
-			$canvas-> fill_ellipse( 
-				$points[$capture], $points[$capture+1], 
+			$canvas-> fill_ellipse(
+				$points[$capture], $points[$capture+1],
 				$aperture, $aperture
 			);
 		}
@@ -68,8 +69,8 @@ my $w = Prima::MainWindow-> create(
 
 		my $i;
 		for ( $i = 0; $i < @points; $i+=2) {
-			$canvas-> ellipse( 
-				$points[$i], $points[$i+1], 
+			$canvas-> ellipse(
+				$points[$i], $points[$i+1],
 				$aperture, $aperture
 			);
 		}
@@ -83,7 +84,7 @@ my $w = Prima::MainWindow-> create(
 		my $i;
 		$capture = undef;
 		for ( $i = 0; $i < @points; $i+=2) {
-			if ( $points[$i] > $x - $aperture && $points[$i] < $x + $aperture && 
+			if ( $points[$i] > $x - $aperture && $points[$i] < $x + $aperture &&
 				$points[$i+1] > $y - $aperture && $points[$i+1] < $y + $aperture) {
 				$capture = $i;
 				last;
