@@ -401,7 +401,7 @@ img_polyline( Handle dest, int n_points, Point * points, PImgPaintContext ctx)
 		delta_y = b.y - a.y;
 		delta_x = b.x - a.x;
 		if (abs(delta_y) > abs(delta_x)) dir = 1;
-		
+
 		if (dir) {
 			curr_maj = a.y;
 			curr_min = a.x;
@@ -413,22 +413,22 @@ img_polyline( Handle dest, int n_points, Point * points, PImgPaintContext ctx)
 			curr_min = a.y;
 			to_maj = b.x;
 			delta_maj = delta_x;
-			delta_min = delta_y;   
+			delta_min = delta_y;
 		}
 
 		if (delta_maj != 0)
 			inc_maj = (abs(delta_maj)==delta_maj ? 1 : -1);
 		else
 			inc_maj = 0;
-		
+
 		if (delta_min != 0)
 			inc_min = (abs(delta_min)==delta_min ? 1 : -1);
 		else
 			inc_min = 0;
-		
+
 		delta_maj = abs(delta_maj);
 		delta_min = abs(delta_min);
-		
+
 		d      = (delta_min << 1) - delta_maj;
 		d_inc1 = (delta_min << 1);
 		d_inc2 = ((delta_min - delta_maj) << 1);
@@ -437,10 +437,10 @@ img_polyline( Handle dest, int n_points, Point * points, PImgPaintContext ctx)
 		while(1) {
 			ox = x;
 			if (dir) {
-				x = curr_min;	 
+				x = curr_min;
 				y = curr_maj;
 			} else {
-				x = curr_maj;	 
+				x = curr_maj;
 				y = curr_min;
 			}
 			if ( acc_y != y ) {
@@ -449,7 +449,6 @@ img_polyline( Handle dest, int n_points, Point * points, PImgPaintContext ctx)
 				acc_x = x;
 				acc_y = y;
 			}
-	
 			if (curr_maj == to_maj) break;
 			curr_maj += inc_maj;
 			if (d < 0) {
@@ -551,7 +550,6 @@ img_polyline2patterns( NPoint * points, int n_points, Byte *lj_hints, double lin
 	float pattern_buf[256], *pattern, sqrt_table[1024];
 	semistatic_t pattern_array;
 	Bool ok = false, closed, strokecolor, new_point, new_stroke, black, joiner;
-	Bool pivot_detected, pivot_registered;
 	int step;
 	float advance, strokelen, pixlen, draw, plotted;
 	double dx, dy;
@@ -580,7 +578,7 @@ img_polyline2patterns( NPoint * points, int n_points, Byte *lj_hints, double lin
 
 	closed = points[0].x == points[n_points-1].x && points[0].y == points[n_points-1].y;
 	i = step = 0;
-	strokecolor = joiner = pivot_detected = pivot_registered = false;
+	strokecolor = joiner = false;
 	new_point = new_stroke = true;
 	advance = strokelen = 0.0;
 	last_a = a = points[0];
@@ -615,7 +613,6 @@ img_polyline2patterns( NPoint * points, int n_points, Byte *lj_hints, double lin
 				/* printf("NEW (%p) <- %p [%p]\n", curr->prev, curr, dst); */
 				last_a = a;
 				last_b = b;
-				pivot_registered = false;
 			}
 		}
 
@@ -672,14 +669,6 @@ img_polyline2patterns( NPoint * points, int n_points, Byte *lj_hints, double lin
 			black = strokecolor;
 		}
 		next_seg_advance = black ? line_width - 1.0 : 1.0;
-		if ( pivot_detected ) {
-			if ( !pivot_registered ) {
-				last_b = b;
-				/* printf("pivot %g.%g\n", b.x, b.y); */
-				pivot_registered = true;
-			}
-			pivot_detected = false;
-		}
 
 #define ADD_POINT_ENTRY(xx)                                                \
 	if ( curr->n_points == 0 ||                                        \
@@ -721,7 +710,7 @@ img_polyline2patterns( NPoint * points, int n_points, Byte *lj_hints, double lin
 		} else if ( draw == pixlen ) {
 			/* exact match that ends line by pattern end */
 			ADD_POINT(a1,b);
-			new_stroke = new_point = pivot_detected = true;
+			new_stroke = new_point = true;
 			advance += (advance > 0.0) ? -draw : next_seg_advance;
 			joiner = black;
 		} else if ( black && draw == 1.0 && pixlen <= 0 ) {
@@ -758,7 +747,7 @@ img_polyline2patterns( NPoint * points, int n_points, Byte *lj_hints, double lin
 	if ( closed && pattern[0] > 1.0 && strokelen > 1.0 && curr != dst ) {
 		NPolyPolyline *p = dst;
 		dst = dst->next;
-		if ( curr->n_points > curr-> size - p-> n_points) {
+		if ( p->n_points > curr-> size - curr-> n_points) {
 			if ( !( curr = nppl_alloc(curr, lj_hints != NULL, curr->size + p-> n_points)))
 				goto EXIT;
 		}
