@@ -68,7 +68,11 @@ XS(Utils_nearest_i_FROMPERL);
 XS(Utils_nearest_d_FROMPERL);
 XS(Utils_stat_FROMPERL);
 XS(Utils_closedir_FROMPERL);
+
 XS(Prima_array_deduplicate_FROMPERL);
+XS(Prima_array_multiply_FROMPERL);
+XS(Prima_array_FETCH_FROMPERL);
+XS(Prima_array_STORE_FROMPERL);
 
 extern PRGBColor prima_read_palette( int * palSize, SV * palette);
 extern Bool prima_read_point( SV *rvav, int * pt, int number, char * error);
