@@ -237,7 +237,7 @@ typedef struct _RotatedFont {
 	struct        RotatedFont *next;
 } RotatedFont, *PRotatedFont;
 
-#define MAX_CACHED_FONTS 50
+#define MAX_CACHED_FONTS 512
 
 typedef struct CachedFont {
 	Byte         type;
@@ -988,7 +988,7 @@ typedef struct _drawable_sys_data
 #define XF_IN_PAINT(x)  ((x)->flags.paint)
 #define XF_LAYERED(x)  ((x)->flags.layered)
 #define XFLUSH          if (XX->flags.force_flush) XFlush(DISP)
-#define XRENDER_SYNC    if (XX->flags.xrender_sync) XSync(DISP, XX->flags.xrender_sync = false)
+#define XRENDER_SYNC    if (XX->flags.xrender_sync) { XFlush(DISP); XX->flags.xrender_sync = false; }
 #define XRENDER_SYNC_NEEDED XX->flags.xrender_sync = true
 
 typedef struct _PaintState
@@ -1014,7 +1014,6 @@ typedef struct _PaintState
 	FillPattern fill_pattern;
 	Font font;
 	unsigned char *dashes;
-	Matrix matrix;
 
 	unsigned int user_data_size;
 	GCStorageFunction * user_destructor;
