@@ -1,4 +1,4 @@
-/***********************************************************/
+/******************************************font->*****************/
 /*                                                         */
 /*  System dependent graphics (unix, x11)                  */
 /*                                                         */
@@ -125,7 +125,7 @@ gp_text_out_rotated(
 
 	if ( !prima_update_rotated_fonts( XX-> font,
 		text, len, wide,
-		PDrawable( self)-> font. direction, 
+		CDrawable( self)-> trig_cache(self),
 		MY_MATRIX,
 		&r, ok_to_not_rotate
 	))
@@ -175,11 +175,11 @@ gp_text_out_rotated(
 		dsy = REVERT( y + ry. i. i) + psy - r-> dimension. y + 1;
 
 		if ( guts. debug & DEBUG_FONTS) {
-			_debug("shift %d %d\n", r-> shift.x, r-> shift.y);
-			_debug("point ref: %d %d => %d %d. dims: %d %d, [%d %d %d]\n", px, py, psx, psy, r-> dimension.x, r-> dimension.y,
+			_debug("shift %d %d", r-> shift.x, r-> shift.y);
+			_debug("point ref: %d %d => %d %d. dims: %d %d, [%d %d %d]", px, py, psx, psy, r-> dimension.x, r-> dimension.y,
 				cs-> lbearing, cs-> rbearing - cs-> lbearing, cs-> width - cs-> rbearing);
-			_debug("plot ref: %d %d => %d %d\n", ax, ay, rx.i.i, ry.i.i);
-			_debug("at: %d %d ( sz = %d), dest: %d %d\n", x, y, XX-> size.y, dsx, dsy);
+			_debug("plot ref: %d %d => %d %d", ax, ay, rx.i.i, ry.i.i);
+			_debug("at: %d %d ( sz = %d), dest: %d %d", x, y, XX-> size.y, dsx, dsy);
 		}
 
 /*   GXandReverse   ropNotDestAnd */		/* dest = (!dest) & src */
@@ -257,15 +257,17 @@ gp_text_out_rotated(
 
 	if ( PDrawable( self)-> font. style & (fsUnderlined|fsStruckOut)) {
 		int lw = 1;
-		int tw = gp_get_text_width( self, text, len, flags | toAddOverhangs) - 1;
-		int d  = XX-> font-> underlinePos;
+		int tw = gp_get_text_width( self, text, len, flags);
+		int d  = PDrawable(self)-> font.underlinePosition;
+		int t  = PDrawable(self)-> font.underlineThickness;
 		Point ovx = gp_get_text_overhangs( self, text, len, flags);
 		int x1, y1, x2, y2;
 
-		if ( lw != XX-> font-> underlineThickness) {
+		if ( lw != t) {
 			XGCValues gcv;
-			lw = gcv.line_width = XX-> font-> underlineThickness;
-			XChangeGC( DISP, XX-> gc, GCLineWidth, &gcv);
+			lw = gcv.line_width = t;
+			gcv.cap_style = CapNotLast;
+			XChangeGC( DISP, XX-> gc, GCLineWidth | GCCapStyle, &gcv);
 		}
 
 		if ( PDrawable( self)-> font. style & fsUnderlined) {
@@ -277,13 +279,14 @@ gp_text_out_rotated(
 			tw += ovx.y;
 			rx. l = tw * r-> matrix[0]. l + ay * r-> matrix[2]. l + 0.5;
 			ry. l = tw * r-> matrix[1]. l + ay * r-> matrix[3]. l + 0.5;
+			tw -= ovx.y;
 			x2 = x + rx. i. i;
 			y2 = y + ry. i. i;
 			XDrawLine( DISP, XX-> gdrawable, XX-> gc, x1, REVERT( y1), x2, REVERT( y2));
 		}
 
 		if ( PDrawable( self)-> font. style & fsStruckOut) {
-			ay = (PDrawable( self)-> font.ascent - PDrawable( self)-> font.internalLeading)/2 +
+			ay = (PDrawable( self)-> font.ascent - PDrawable( self)-> font.internalLeading)/3 +
 				+ ( XX-> flags. base_line ? 0 : XX-> font-> font. descent);
 			rx. l = -ovx.x * r-> matrix[0]. l + ay * r-> matrix[2]. l + 0.5;
 			ry. l = -ovx.x * r-> matrix[1]. l + ay * r-> matrix[3]. l + 0.5;
@@ -292,6 +295,7 @@ gp_text_out_rotated(
 			tw += ovx.y;
 			rx. l = tw * r-> matrix[0]. l + ay * r-> matrix[2]. l + 0.5;
 			ry. l = tw * r-> matrix[1]. l + ay * r-> matrix[3]. l + 0.5;
+			tw -= ovx.y;
 			x2 = x + rx. i. i;
 			y2 = y + ry. i. i;
 			XDrawLine( DISP, XX-> gdrawable, XX-> gc, x1, REVERT( y1), x2, REVERT( y2));
@@ -349,13 +353,15 @@ draw_text_underline(Handle self, const char * text, int x, int y, int len, int f
 {
 	DEFXX;
 	int lw = 1;
-	int tw = gp_get_text_width( self, text, len, flags | toAddOverhangs);
-	int d  = XX-> font-> underlinePos;
+	int tw = gp_get_text_width( self, text, len, flags);
+	int d  = PDrawable(self)-> font.underlinePosition;
+	int t  = PDrawable(self)-> font.underlineThickness;
 	Point ovx = gp_get_text_overhangs( self, text, len, flags);
-	if ( lw != XX-> font-> underlineThickness) {
+	if ( lw != t ) {
 		XGCValues gcv;
-		lw = gcv.line_width = XX-> font-> underlineThickness;
-		XChangeGC( DISP, XX-> gc, GCLineWidth, &gcv);
+		lw = gcv.line_width = t;
+		gcv.cap_style = CapNotLast;
+		XChangeGC( DISP, XX-> gc, GCLineWidth | GCCapStyle, &gcv);
 	}
 	if ( !XX-> flags. base_line)
 		y += XX-> font-> font. descent;
@@ -363,7 +369,7 @@ draw_text_underline(Handle self, const char * text, int x, int y, int len, int f
 		XDrawLine( DISP, XX-> gdrawable, XX-> gc,
 			x - ovx.x, REVERT( y + d), x + tw - 1 + ovx.y, REVERT( y + d));
 	if ( PDrawable( self)-> font. style & fsStruckOut) {
-		int scy = REVERT( y + (XX-> font-> font.ascent - XX-> font-> font.internalLeading)/2);
+		int scy = REVERT( y + (XX-> font-> font.ascent - XX-> font-> font.internalLeading)/3);
 		XDrawLine( DISP, XX-> gdrawable, XX-> gc,
 			x - ovx.x, scy, x + tw - 1 + ovx.y, scy);
 	}
@@ -587,8 +593,25 @@ prima_text_shaper_core_text( Handle self, PTextShapeRec r)
 }
 
 PTextShapeFunc
-apc_gp_get_text_shaper( Handle self, int * type)
+apc_font_get_text_shaper( Handle self, int * type)
 {
+
+#ifdef USE_FONTQUERY
+	if ( is_opt(optInFontQuery) ) {
+		int t = *type;
+		if (!(X(self)->font))
+			return NULL;
+#ifdef WITH_HARFBUZZ
+		if ( guts. use_harfbuzz && *type == tsFull )
+			return prima_fq_text_shaper_harfbuzz;
+#endif
+		*type = tsGlyphs;
+		return ( t == tsBytes ) ?
+			prima_fq_text_shaper_bytes :
+			prima_fq_text_shaper_ident;
+	}
+#endif
+
 #ifdef USE_XFT
 	if ( X(self)-> font && X(self)-> font-> xft) {
 		int t = *type;
@@ -602,12 +625,13 @@ apc_gp_get_text_shaper( Handle self, int * type)
 			prima_xft_text_shaper_ident;
 	}
 #endif
+
 	*type = tsNone;
 	return prima_text_shaper_core_text;
 }
 
 PFontABC
-prima_xfont2abc( XFontStruct * fs, int firstChar, int lastChar)
+prima_corefont_xfont2abc( XFontStruct * fs, int firstChar, int lastChar)
 {
 	int k, l;
 	CharStructABC s;
@@ -623,25 +647,7 @@ prima_xfont2abc( XFontStruct * fs, int firstChar, int lastChar)
 }
 
 PFontABC
-apc_gp_get_font_abc( Handle self, int firstChar, int lastChar, int flags)
-{
-	PFontABC abc;
-
-	if ( self) {
-		DEFXX;
-#ifdef USE_XFT
-		if ( XX-> font-> xft)
-			return prima_xft_get_font_abc( self, firstChar, lastChar, flags);
-#endif
-
-		abc = prima_xfont2abc( XX-> font-> fs, firstChar, lastChar);
-	} else
-		abc = prima_xfont2abc( guts. font_abc_nil_hack, firstChar, lastChar);
-	return abc;
-}
-
-PFontABC
-prima_xfont2def( Handle self, int first, int last)
+prima_corefont_xfont2def( Handle self, int first, int last)
 {
 	DEFXX;
 	XCharStruct *max;
@@ -725,16 +731,52 @@ prima_xfont2def( Handle self, int first, int last)
 	return ret;
 }
 
+PFontABC
+apc_gp_get_font_abc( Handle self, int firstChar, int lastChar, int flags)
+{
+	PFontABC abc;
+
+#ifdef USE_FONTQUERY
+	if ( self && is_opt(optInFontQuery) ) {
+		if (X(self)->font)
+			return prima_fq_get_font_abc( self, firstChar, lastChar, flags);
+		return NULL;
+	}
+#endif
+
+	if ( self) {
+		DEFXX;
+#ifdef USE_XFT
+		if ( XX-> font-> xft)
+			return prima_xft_get_font_abc( self, firstChar, lastChar, flags);
+#endif
+
+		abc = prima_corefont_xfont2abc( XX-> font-> fs, firstChar, lastChar);
+	} else
+		abc = prima_corefont_xfont2abc( guts. font_abc_nil_hack, firstChar, lastChar);
+	return abc;
+}
+
 PFontABC
 apc_gp_get_font_def( Handle self, int firstChar, int lastChar, int flags)
 {
 	PFontABC abc;
+
+#ifdef USE_FONTQUERY
+	if ( is_opt(optInFontQuery) ) {
+		if (X(self)->font)
+			return prima_fq_get_font_def( self, firstChar, lastChar, flags);
+		return NULL;
+	}
+#endif
+
 #ifdef USE_XFT
 	DEFXX;
 	if ( XX-> font-> xft)
 		return prima_xft_get_font_def( self, firstChar, lastChar, flags);
 #endif
-	abc = prima_xfont2def( self, firstChar, lastChar);
+
+	abc = prima_corefont_xfont2def( self, firstChar, lastChar);
 	return abc;
 }
 
@@ -761,10 +803,18 @@ apc_gp_get_text_width( Handle self, const char * text, int len, int flags)
 	flags &= ~toGlyphs;
 	if ( len > 65535 ) len = 65535;
 
+#ifdef USE_FONTQUERY
+	if ( is_opt(optInFontQuery) ) {
+		if ( X(self)->font)
+			return prima_fq_get_text_width( self, text, len, flags, NULL);
+		return 0;
+	}
+#endif
+
 #ifdef USE_XFT
 	if ( X(self)-> font-> xft)
 		return prima_xft_get_text_width( X(self)-> font, text, len, flags,
-			X(self)-> xft_map8, NULL);
+			X(self)-> fc_map8, NULL);
 #endif
 
 	if ( flags & toUTF8 )
@@ -781,6 +831,14 @@ apc_gp_get_glyphs_width( Handle self, PGlyphsOutRec t)
 	int ret;
 	if ( t->len > 65535 ) t->len = 65535;
 
+#ifdef USE_FONTQUERY
+	if ( is_opt(optInFontQuery) ) {
+		if ( X(self)->font)
+			return prima_fq_get_glyphs_width( self, t, NULL);
+		return 0;
+	}
+#endif
+
 #ifdef USE_XFT
 	if ( X(self)-> font-> xft)
 		return prima_xft_get_glyphs_width( self, X(self)-> font, t, NULL);
@@ -792,49 +850,36 @@ apc_gp_get_glyphs_width( Handle self, PGlyphsOutRec t)
 	return ret;
 }
 
+Point *
+prima_get_text_box( Handle self, Point ovx, int advance )
+{
+	DEFXX;
+	Point *pt;
+
+	if ( !( pt = ( Point *) malloc( sizeof( Point) * 5)))
+		return NULL;
+	ovx.x = (ovx.x < 0) ? 0 : -ovx.x;
+	ovx.y = (ovx.y < 0) ? 0 : -ovx.y;
+	Drawable_calculate_text_box( self, advance, XX->flags.base_line, ovx, pt);
+	return pt;
+}
+
 static Point *
 gp_get_text_box( Handle self, const char * text, int len, int flags)
 {
 	DEFXX;
-	Point * pt = ( Point *) malloc( sizeof( Point) * 5);
 	int x;
 	Point ovx;
 
-	if ( !pt) return NULL;
 	if ( flags & toGlyphs ) flags &= ~toUTF8;
-
 	x = (flags & toUTF8) ?
 		XTextWidth16( XX-> font-> fs, ( XChar2b*) text, len) :
 		XTextWidth( XX-> font-> fs, (char*)text, len);
 	ovx = gp_get_text_overhangs( self, text, len, flags);
-
-	pt[0].y = pt[2]. y = XX-> font-> font. ascent - 1;
-	pt[1].y = pt[3]. y = - XX-> font-> font. descent;
-	pt[4].y = 0;
-	pt[4].x = x;
-	pt[3].x = pt[2]. x = x + ovx. y;
-	pt[0].x = pt[1]. x = - ovx. x;
-
-	if ( !XX-> flags. base_line) {
-		int i;
-		for ( i = 0; i < 4; i++) pt[i]. y += XX-> font-> font. descent;
-	}
-
-	if ( PDrawable( self)-> font. direction != 0) {
-		int i;
-		double s = sin( PDrawable( self)-> font. direction / 57.29577951);
-		double c = cos( PDrawable( self)-> font. direction / 57.29577951);
-		for ( i = 0; i < 5; i++) {
-			double x = pt[i]. x * c - pt[i]. y * s;
-			double y = pt[i]. x * s + pt[i]. y * c;
-			pt[i]. x = x + (( x > 0) ? 0.5 : -0.5);
-			pt[i]. y = y + (( y > 0) ? 0.5 : -0.5);
-		}
-	}
-
-	return pt;
+	return prima_get_text_box(self, ovx, x);
 }
 
+
 Point *
 apc_gp_get_text_box( Handle self, const char * text, int len, int flags)
 {
@@ -842,10 +887,19 @@ apc_gp_get_text_box( Handle self, const char * text, int len, int flags)
 
 	if ( len > 65535 ) len = 65535;
 
+#ifdef USE_FONTQUERY
+	if ( is_opt(optInFontQuery) ) {
+		if ( X(self)->font)
+			return prima_fq_get_text_box( self, text, len, flags);
+		return NULL;
+	}
+#endif
+
 #ifdef USE_XFT
 	if ( X(self)-> font-> xft)
 		return prima_xft_get_text_box( self, text, len, flags);
 #endif
+
 	if ( flags & toUTF8)
 		if ( !( text = ( char *) prima_alloc_utf8_to_wchar( text, len))) return 0;
 	ret = gp_get_text_box( self, text, len, flags);
@@ -859,10 +913,20 @@ apc_gp_get_glyphs_box( Handle self, PGlyphsOutRec t)
 {
 	Point * ret;
 	if ( t->len > 65535 ) t->len = 65535;
+
+#ifdef USE_FONTQUERY
+	if ( is_opt(optInFontQuery) ) {
+		if ( X(self)->font)
+			return prima_fq_get_glyphs_box( self, t);
+		return NULL;
+	}
+#endif
+
 #ifdef USE_XFT
 	if ( X(self)-> font-> xft)
 		return prima_xft_get_glyphs_box( self, t);
 #endif
+
 	SWAP_BYTES(t->glyphs,t->len);
 	ret = gp_get_text_box( self, (char*) t->glyphs, t->len, toUTF8);
 	SWAP_BYTES(t->glyphs,t->len);
